from langchain.llms.base import LLM
from langchain.callbacks.manager import CallbackManagerForLLMRun
from pydantic import root_validator
from typing import Dict, List, Optional
from transformers import PreTrainedModel, PreTrainedTokenizer


class WrapperLLM(LLM):
    tokenizer: PreTrainedTokenizer = None
    model: PreTrainedModel = None

    @root_validator()
    def validate_environment(self, values: Dict) -> Dict:
        """Validate the environment."""
        # print(values)
        if values.get("model") is None:
            raise ValueError("No model provided.")
        if values.get("tokenizer") is None:
            raise ValueError("No tokenizer provided.")
        return values

    def _call(self, prompt: str, stop: Optional[List[str]] = None,
              run_manager: Optional[CallbackManagerForLLMRun] = None, **kwargs) -> str:
        resp, his = self.model.chat(self.tokenizer, prompt)
        return resp

    @property
    def _llm_type(self) -> str:
        """Return type of llm."""
        return "wrapper"
